import React, { Component } from "react"
import cn from "classnames"

class ReactimaDropdown extends Component {
  constructor(props) {
    super(props)
    this.state = { title: null, open: false }
  }

  componentDidMount() {
    document.addEventListener("click", this.handleDocumentClick.bind(this))
  }

  componentWillUnmount() {
    document.removeEventListener("click", this.handleDocumentClick.bind(this))
  }

  toogleOpen() {
    if (this.state.open) {
      this.setState({
        open: false
      })
    } else {
      this.setState({
        open: true
      })
    }
  }

  handleDocumentClick() {
    const wasDown = this.mouseDownOnModal
    const wasUp = this.mouseUpOnModal
    this.mouseDownOnModal = false
    this.mouseUpOnModal = false

    if (!wasDown && !wasUp && this.state.open) {
      this.toogleOpen()
    }
  }

  handleMouseDown() {
    this.mouseDownOnModal = true
  }

  handleMouseUp() {
    this.mouseUpOnModal = true
  }

  render() {
    return (
      <div
        className="dropdown"
        onMouseDown={this.handleMouseDown.bind(this)}
        onMouseUp={this.handleMouseUp.bind(this)}
      >
        <a
          className="dropdown-toggle link"
          onClick={this.toogleOpen.bind(this)}
        >
          {this.state.title}
        </a>
        <div
          className={cn(
            "dropdown-menu",
            this.props.rightSide ? "dropdown-menu-right" : "",
            { show: this.state.open }
          )}
        >
          {this.props.items.map(
            (item, key) =>
              (!this.props.isDivider
                ? <a
                    key={key}
                    className="dropdown-item"
                    tabIndex="-1"
                    href="#"
                    value={item.value}
                    onClick={e => {
                      e.preventDefault()
                      if (!this.props.disabled) {
                        this.setState({
                          title: e.target.text,
                          open: false
                        })
                        this.props.onSelect({
                          text: e.target.text,
                          value: e.target.getAttribute("value")
                        })
                      }
                    }}
                  >
                    {item.text}
                  </a>
                : <div key={key} className="dropdown-divider" />)
          )}
        </div>
      </div>
    )
  }
}

ReactimaDropdown.defaultProps = {
  title: "Dropdown",
  disabled: false,
  updateTitle: true
}

export default ReactimaDropdown
