import React, { Component } from "react"
import { bindActionCreators } from "redux"
import { connect } from "react-redux"

import * as authActions from "../../actions/auth"

import Autosuggest from "./ReactimaSuggest"

const languages = [
  {
    title: "Candidates",
    languages: [
      {
        name: "Ilya Sidorov",
        year: 1972
      }
    ]
  },
  {
    title: "Positions",
    languages: [
      {
        name: "C++",
        year: 1983
      },
      {
        name: "Perl",
        year: 1987
      }
    ]
  },
  {
    title: "Positions",
    languages: [
      {
        name: "Haskell",
        year: 1990
      },
      {
        name: "Python",
        year: 1991
      },
      {
        name: "Java",
        year: 1995
      },
      {
        name: "Javascript",
        year: 1995
      },
      {
        name: "PHP",
        year: 1995
      },
      {
        name: "Ruby",
        year: 1995
      }
    ]
  },
  {
    title: "2000s",
    languages: [
      {
        name: "C#",
        year: 2000
      },
      {
        name: "Scala",
        year: 2003
      },
      {
        name: "Clojure",
        year: 2007
      },
      {
        name: "Go",
        year: 2009
      }
    ]
  },
  {
    title: "2010s",
    languages: [
      {
        name: "Elm",
        year: 2012
      }
    ]
  }
]

const getSuggestions = value => {
  const escapedValue = value.trim() //TODO escapeRegexCharacters(value.trim());

  if (escapedValue === "") {
    return []
  }

  const regex = new RegExp("^" + escapedValue, "i")

  return languages
    .map(section => {
      return {
        title: section.title,
        languages: section.languages.filter(language =>
          regex.test(language.name)
        )
      }
    })
    .filter(section => section.languages.length > 0)
}

class Search extends Component {
  constructor() {
    super()
    this.state = {
      value: "",
      suggestions: []
    }
  }

  onChange = (event, { newValue }) => {
    this.setState({
      value: newValue
    })
  }

  onSuggestionsFetchRequested = ({ value }) => {
    setTimeout(() => {
      this.setState({
        suggestions: getSuggestions(value)
      })
    }, 500)
  }

  onSuggestionsClearRequested = () => {
    this.setState({
      suggestions: []
    })
  }

  render() {
    const { value, suggestions } = this.state
    const inputProps = {
      placeholder: "",
      value,
      onChange: this.onChange
    }

    return (
      <Autosuggest
        renderInputComponent={props => (
          <div className="input-group">
            <div className="input-group-addon hh-search-pre">
              <i className="fa fa-search" aria-hidden="true" />
            </div>
            <input
              id="search"
              type="text"
              placeholder="Search"
              {...props}
              className="form-control hh-warning-border hh-search"
            />
          </div>
        )}
        multiSection={true}
        suggestions={suggestions}
        onSuggestionsFetchRequested={this.onSuggestionsFetchRequested}
        onSuggestionsClearRequested={this.onSuggestionsClearRequested}
        getSuggestionValue={suggestion => suggestion.name}
        renderSuggestion={suggestion => <span>{suggestion.name}</span>}
        renderSectionTitle={section => <strong>{section.title}</strong>}
        getSectionSuggestions={section => section.languages}
        inputProps={inputProps}
        focusFirstSuggestion={true}
        focusInputOnSuggestionClick={false}
        id="multiple-sections-example"
      />
    )
  }
}

function mapStateToProps(state) {
  return {
    token: state.auth.token
  }
}

function mapDispatchToProps(dispatch) {
  return {
    auth: bindActionCreators(authActions, dispatch)
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(Search)
